<?

require_once( dirname(__FILE__).'/../LanguageConverter.php' ); 
require_once( dirname(__FILE__).'/LanguageKk_cyrl.php' ); 

/** Kyrgyz (Кыргызча)  - ARABIC TO CYRYLLIC CONVERTOR
* WARNING:
* Test version only. Just ARAB definy line was replaced to KY_C_UC / KY_C_LC  
*
* Problematic letters:
* 
* Not a serious problem: letters Ъ ъ, Ь ь have no Arabic equivalent, used only in loan words
* 
* Not very serious problem: Arabic letters have no small / CAPITAL letters, 
* so all text will be in Cyrillic with small letters only.
* 
* Serious problem - letters Ы ы, И и have the same Arabic equivalent "ى"  - no idea how 
* to definy rules when ى should be ы and when и
* For now converted as И и
* 
* Serious problem - letters Э э, Е е have the same Arabic equivalent "ە" - no idea how 
* to definy rules when ە should be Е е and when Э э
* For now converted as Е е
* * 
*/ 

//define( 'KY_C_UC', 'АБВГДЕЁЖЗИЙКЛМНҢОӨПРСТУҮФХЦЧШЩЪЫЬЭЮЯ' ); # Kyrgyz Cyrillic uppercase 
//define( 'KY_C_LC', 'абвгдеёжзийклмнңоөпрстуүфхцчшщъыьэюя' ); # Kyrgyz Cyrillic lowercase 
define( 'KY_L_UC', 'ABCÇDEÉFGĞHIİJKLMNÑOÖPQRSŞTUÜVYÝZ' ); # Kyrgyz Latin uppercase 
define( 'KY_L_LC', 'abcçdeéfgğhıijklmnñoöpqrsştuüvyýz' ); # Kyrgyz Latin lowercase 
define( 'KY_C_LC', 'ٴابپتجحدرزسشعفقكلمنڭەوۇۋۆىيچھ' ); # Kyrgyz Arabic 
define( 'KY_C_UC', 'ٴابپتجحدرزسشعفقكلمنڭەوۇۋۆىيچھ' ); # Kyrgyz Arabic
define( 'H_HAMZA', 'ٴ' ); # U+0674 ARABIC LETTER HIGH HAMZA 
//define( 'ZWNJ', '‌' ); # U+200C ZERO WIDTH NON-JOINER 

/** Kyrgyz (Кыргызча) 
* converter routines 
* 
* @ingroup Language 
*/ 
class KkConverter extends LanguageConverter { 

function __construct($langobj, $maincode, 
$variants=array(), 
$variantfallbacks=array(), 
$flags = array()) { 
parent::__construct( $langobj, $maincode, 
$variants, $variantfallbacks, $flags ); 

// No point delaying this since they're in code. 
// Waiting until loadDefaultTables() means they never get loaded 
// when the tables themselves are loaded from cache. 
$this->loadRegs(); 
} 

function loadDefaultTables() { 
// require( dirname(__FILE__)."/../../includes/KyConversion.php" ); 
// Placeholder for future implementing. Remove variables declarations 
// after generating KyConversion.php 
$ky2Cyrl = array(); 
$ky2Latn = array(); 
$ky2Arab = array(); 
$ky2KZ = array(); 
$ky2TR = array(); 
$ky2CN = array(); 

$this->mTables = array( 
'ky-cyrl' => new ReplacementArray( $ky2Cyrl ), 
'ky-latn' => new ReplacementArray( $ky2Latn ), 
'ky-arab' => new ReplacementArray( $ky2Arab ), 
'ky-kgz' => new ReplacementArray( array_merge($ky2Cyrl, $ky2KZ) ), 
'ky-tr' => new ReplacementArray( array_merge($ky2Latn, $ky2TR) ), 
'ky-cn' => new ReplacementArray( array_merge($ky2Arab, $ky2CN) ), 
'ky' => new ReplacementArray() 
); 
} 

function postLoadTables() { 
$this->mTables['ky-kgz']->merge( $this->mTables['ky-cyrl'] ); 
$this->mTables['ky-tr']->merge( $this->mTables['ky-latn'] ); 
$this->mTables['ky-cn']->merge( $this->mTables['ky-arab'] ); 
} 

function loadRegs() { 

$this->mCyrl2Latn = array( 
## Punctuation 
'/№/u' => 'No.', 
## Е after vowels 
'/([АЕЁИОӨҮЭЮЯЪЬ])Е/u' => '$1YE', 
'/([АЕЁИОӨҮЭЮЯЪЬ])е/ui' => '$1ye', 
## leading ЁЮЯЩ 
'/^Ё(['.KY_C_UC.']|$)/u' => 'YO$1', '/^Ё(['.KY_C_LC.']|$)/u' => 'Yo$1', 
'/^Ю(['.KY_C_UC.']|$)/u' => 'YU$1', '/^Ю(['.KY_C_LC.']|$)/u' => 'Yu$1', 
'/^Я(['.KY_C_UC.']|$)/u' => 'YA$1', '/^Я(['.KY_C_LC.']|$)/u' => 'Ya$1', 
'/^Щ(['.KY_C_UC.']|$)/u' => 'ŞÇ$1', '/^Щ(['.KY_C_LC.']|$)/u' => 'Şç$1', 
## other ЁЮЯ 
'/Ё/u' => 'YO', '/ё/u' => 'yo', 
'/Ю/u' => 'YU', '/ю/u' => 'yu', 
'/Я/u' => 'YA', '/я/u' => 'ya', 
'/Щ/u' => 'ŞÇ', '/щ/u' => 'şç', 
## soft and hard signs 
'/[ъЪ]/u' => 'ʺ', '/[ьЬ]/u' => 'ʹ', 
## other characters 
'/А/u' => 'A', '/а/u' => 'a', 
'/Б/u' => 'B', '/б/u' => 'b', '/В/u' => 'V', '/в/u' => 'v', 
'/Г/u' => 'G', '/г/u' => 'g', 
'/Д/u' => 'D', '/д/u' => 'd', '/Е/u' => 'E', '/е/u' => 'e', 
'/Ж/u' => 'J', '/ж/u' => 'j', '/З/u' => 'Z', '/з/u' => 'z', 
'/И/u' => 'İ', '/и/u' => 'i', '/Й/u' => 'Y', '/й/u' => 'y', 
'/К/u' => 'K', '/к/u' => 'k', 
'/Л/u' => 'L', '/л/u' => 'l', '/М/u' => 'M', '/м/u' => 'm', 
'/Н/u' => 'N', '/н/u' => 'n', '/Ң/u' => 'Ñ', '/ң/u' => 'ñ', 
'/О/u' => 'O', '/о/u' => 'o', '/Ө/u' => 'Ö', '/ө/u' => 'ö', 
'/П/u' => 'P', '/п/u' => 'p', '/Р/u' => 'R', '/р/u' => 'r', 
'/С/u' => 'S', '/с/u' => 's', '/Т/u' => 'T', '/т/u' => 't', 
'/У/u' => 'U', '/у/u' => 'u', 
'/Ү/u' => 'Ü', '/ү/u' => 'ü', '/Ф/u' => 'F', '/ф/u' => 'f', 
'/Х/u' => 'H', '/х/u' => 'h', 
'/Ц/u' => 'C', '/ц/u' => 'c', '/Ч/u' => 'Ç', '/ч/u' => 'ç', 
'/Ш/u' => 'Ş', '/ш/u' => 'ş', '/Ы/u' => 'I', '/ы/u' => 'ı', 
'/Э/u' => 'É', '/э/u' => 'é', 
); 

$this->mLatn2Cyrl = array( 
## Punctuation 
'/№/u' => 'No.', 
## Şç 
'/ŞÇʹ/u'=> 'ЩЬ', '/Şçʹ/u'=> 'Щь', '/Şçʹ/u'=> 'Щь', 
'/Ş[Çç]/u' => 'Щ', '/şç/u' => 'щ', 
## soft and hard signs 
'/(['.KY_L_UC.'])ʺ(['.KY_L_UC.'])/u' => '$1Ъ$2', 
'/ʺ(['.KY_L_LC.'])/u' => 'ъ$1', 
'/(['.KY_L_UC.'])ʹ(['.KY_L_UC.'])/u' => '$1Ь$2', 
'/ʹ(['.KY_L_LC.'])/u' => 'ь$1', 
'/ʺ/u' => 'ъ', 
'/ʹ/u' => 'ь', 
## Ye Yo Yu Ya. 
'/Y[Ee]/u' => 'Е', '/ye/u' => 'е', 
'/Y[Oo]/u' => 'Ё', '/yo/u' => 'ё', 
'/Y[UWuw]/u' => 'Ю', '/y[uw]/u' => 'ю', 
'/Y[Aa]/u' => 'Я', '/ya/u' => 'я', 
## other characters 
'/A/u' => 'А', '/a/u' => 'а', 
'/B/u' => 'Б', '/b/u' => 'б', '/C/u' => 'Ц', '/c/u' => 'ц', 
'/Ç/u' => 'Ч', '/ç/u' => 'ч', '/D/u' => 'Д', '/d/u' => 'д', 
'/E/u' => 'Е', '/e/u' => 'е', '/É/u' => 'Э', '/é/u' => 'э', 
'/F/u' => 'Ф', '/f/u' => 'ф', '/G/u' => 'Г', '/g/u' => 'г', 
'/Ğ/u' => 'Г', '/ğ/u' => 'г', '/H/u' => 'Х', '/h/u' => 'х', 
'/I/u' => 'Ы', '/ı/u' => 'ы', '/İ/u' => 'И', '/i/u' => 'и', 
'/J/u' => 'Ж', '/j/u' => 'ж', 
'/K/u' => 'К', '/k/u' => 'к', '/L/u' => 'Л', '/l/u' => 'л', 
'/M/u' => 'М', '/m/u' => 'м', '/N/u' => 'Н', '/n/u' => 'н', 
'/Ñ/u' => 'Ң', '/ñ/u' => 'ң', '/O/u' => 'О', '/o/u' => 'о', 
'/Ö/u' => 'Ө', '/ö/u' => 'ө', '/P/u' => 'П', '/p/u' => 'п', 
'/Q/u' => 'К', '/q/u' => 'к', '/R/u' => 'Р', '/r/u' => 'р', 
'/S/u' => 'С', '/s/u' => 'с', '/Ş/u' => 'Ш', '/ş/u' => 'ш', 
'/T/u' => 'Т', '/t/u' => 'т', '/U/u' => 'У', '/u/u' => 'у', 
'/Ü/u' => 'Ү', '/ü/u' => 'ү', '/V/u' => 'В', '/v/u' => 'в', 
'/Y/u' => 'Й', '/y/u' => 'й', 
'/Z/u' => 'З', '/z/u' => 'з', 
); 

$this->mCyLa2Arab = array( 
// This is to change "ك" for "ق" when АУОЫ is with combination к. E.g. ка, ак, ку, ук, ко, ок, кы, ык
// This is to change "گ" for "ع" when АУОЫ is with combination г. E.g. га, аг, гу, уг, го, ог, гы, ыг 
			'/([АУОЫ])к/ui' => '$1ق',
			'/([АУОЫ])г/ui' => '$1ع',
			'/([АУОЫ])К/ui' => '$1ق',
			'/([АУОЫ])Г/ui' => '$1ع',
			'/к([АУОЫ])/ui' => 'ق$1',
			'/г([АУОЫ])/ui' => 'ع$1',
			'/К([АУОЫ])/ui' => 'ق$1',
			'/Г([АУОЫ])/ui' => 'ع$1',
## Punctuation -> Arabic 
			'/#|№|No\./u' => '؀', # &#x0600;
			'/\،/' => ',', # &#x060C;
			'/؛/'  => ';', # &#x061B;
			'/\؟/' => '?', # &#x061F;
			'/٪/'  => '%', # &#x066A;
			'/\٭/' => '*', # &#x066D;
## Digits -> Arabic 
'/۰/' => '0', # ۰ 
'/۱/' => '1', # ۱ 
'/۲/' => '2', # ۲ 
'/۳/' => '3', # ۳ 
'/۴/' => '4', # ۴ 
'/۵/' => '5', # ۵ 
'/۶/' => '6', # ۶ 
'/۷/' => '7', # ۷ 
'/۸/' => '8', # ۸ 
'/۹/' => '9', # ۹ 
## Cyrillic -> Arabic 
'/Аллаһ/ui' => 'ﷲ', 
'/([АЕЁИОӨҮЭЮЯЪЬ])е/ui' => '$1يە', 
'/([АУОЫ])ك/ui' => '$1ق',
'/[еэ]/ui' => 'ە', '/[ъь]/ui' => '', 
'/[а]/ui' => 'ا', 

// STARE
// '/[оө]/ui' => 'ۅ'
// '/[уү]/ui' => 'ۉ'
// '/я/ui' => 'يا' + ia

// Rozdelil som ө, о - у, ү
'/يو/ui' => 'ё',
'/[ۅ]/ui' => 'ө', '/[و]/ui' => 'о', 
'/[ۇ]/ui' => 'у',
'/[ۉ]/ui' => 'ү',
'/[ى]/ui' => 'ы', 

// Cyrillic -> Arabic MOJE UPRAVY - upravil som yya = iya
'/يا/ui' => 'я',
'/ىيا/ui' => 'ия',


'/[ا]/ui' => 'а',
'/[ە]/ui' => 'е',





## Cyrillic -> Arabic MOJE UPRAVY KONIEC
'/[ى]/ui' => 'и', '/يۋ/ui' => 'ю',  '/[ي]/ui' => 'й',  
'/تس/ui' => 'ц', '/شش/ui' => 'щ', 
'/ح/ui' => 'х', '/چ/ui' => 'ч', 
#'/х/ui' => 'ھ', '/ч/ui' => 'چ', 
'/ب/ui' => 'б', '/ۋ/ui' => 'в', '/گ/ui' => 'г', '/ع/ui' => 'г',
'/د/ui' => 'д', '/ج/ui' => 'ж', '/ز/ui' => 'з', '/ك/ui' => 'к', '/ق/ui' => 'к',
'/ل/ui' => 'л', '/م/ui' => 'м', '/ن/ui' => 'н', 
'/ڭ/ui' => 'ң', '/پ/ui' => 'п', '/ر/ui' => 'р', '/س/ui' => 'с', 
'/ت/ui' => 'т', '/ف/ui' => 'ф', '/ح/ui' => 'х', 
'/ش/ui' => 'ш', 
'/([АУОЫ])ك/ui' => '$1ق',
## Latin -> Arabic // commented for now... 
/*'/Allah/ui' => 'ﷲ', 
'/[eé]/ui' => 'ە', '/[yý]/ui' => 'ي', '/[ʺʹ]/ui' => '', 
'/[aä]/ui' => 'ا', '/[oö]/ui' => 'و', '/[uü]/ui' => 'ۇ', 
'/[iİyY]/ui' => 'ى', '/[ıI]/u' => 'ى', 
'/c/ui' => 'تس', 
'/ç/ui' => 'تش', '/h/ui' => 'ح', 
#'/ç/ui' => 'چ', '/h/ui' => 'ھ', 
'/b/ui' => 'ب','/d/ui' => 'د', 
'/f/ui' => 'ف', '/g/ui' => 'گ', '/ğ/ui' => 'ع', 
'/j/ui' => 'ج', '/k/ui' => 'ك', '/l/ui' => 'ل', '/m/ui' => 'م', 
'/n/ui' => 'ن', '/ñ/ui' => 'ڭ', '/p/ui' => 'پ', '/q/ui' => 'ق', 
'/r/ui' => 'ر', '/s/ui' => 'س', '/ş/ui' => 'ش', '/t/ui' => 'ت', 
'/v/ui' => 'ۆ', '/h/ui' => 'ح', '/z/ui' => 'ز',*/ 
); 
} 

/* rules should be defined as -{ekavian | iyekavian-} -or- 
-{code:text | code:text | ...}- 
update: delete all rule parsing because it's not used 
currently, and just produces a couple of bugs 
*/ 
function parseManualRule($rule, $flags=array()) { 
if(in_array('T',$flags)){ 
return parent::parseManualRule($rule, $flags); 
} 

// otherwise ignore all formatting 
foreach($this->mVariants as $v) { 
$carray[$v] = $rule; 
} 

return $carray; 
} 

/* 
* A function wrapper: 
* - if there is no selected variant, leave the link 
* names as they were 
* - do not try to find variants for usernames 
*/ 
function findVariantLink( &$link, &$nt, $ignoreOtherCond = false ) { 
// check for user namespace 
if(is_object($nt)){ 
$ns = $nt->getNamespace(); 
if($ns==NS_USER || $ns==NS_USER_TALK) 
return; 
} 

$oldlink=$link; 
parent::findVariantLink( $link, $nt, $ignoreOtherCond ); 
if( $this->getPreferredVariant()==$this->mMainLanguageCode ) 
$link=$oldlink; 
} 

/* 
* An ugly function wrapper for parsing Image titles 
* (to prevent image name conversion) 
*/ 
function autoConvert($text, $toVariant=false) { 
global $wgTitle; 
if(is_object($wgTitle) && $wgTitle->getNameSpace()==NS_FILE){ 
$imagename = $wgTitle->getNsText(); 
if(preg_match("/^$imagename:/",$text)) return $text; 
} 
return parent::autoConvert($text,$toVariant); 
} 

/** 
* It translates text into variant 
*/ 
function translate( $text, $toVariant ){ 
global $wgContLanguageCode; 
$text = parent::translate( $text, $toVariant ); 

$letters = ''; 
switch( $toVariant ) { 
case 'ky-cyrl': 
case 'ky-kgz': 
$letters = KY_L_UC . KY_L_LC . 'ʺʹ#0123456789'; 
$wgContLanguageCode = 'ky'; 
break; 
case 'ky-latn': 
case 'ky-tr': 
$letters = KY_C_UC . KY_C_LC . '№0123456789'; 
$wgContLanguageCode = 'ky-Latn'; 
break; 
case 'ky-arab': 
case 'ky-cn': 
$letters = KY_C_UC.KY_C_LC./*KY_L_UC.KY_L_LC.'ʺʹ'.*/',;\\?%\\*№0123456789'; 
$wgContLanguageCode = 'ky-Arab'; 
break; 
default: 
$wgContLanguageCode = 'ky'; 
return $text; 
} 
		// disable conversion variables like $1, $2...
		$varsfix = '\$[0-9]';

		$matches = preg_split( '/' . $varsfix . '[^' . $letters . ']+/u', $text, -1, PREG_SPLIT_OFFSET_CAPTURE);
		$mstart = 0;
		$ret = '';
		foreach( $matches as $m ) {
			$ret .= substr( $text, $mstart, $m[1]-$mstart );
			$ret .= $this->regsConverter( $m[0], $toVariant );
			$mstart = $m[1] + strlen($m[0]);
		}
		return $ret;
	}

function regsConverter( $text, $toVariant ) { 
if ($text == '') return $text; 

$pat = array(); 
$rep = array(); 
switch( $toVariant ) { 
case 'ky-arab': 
case 'ky-cn': 
$letters = KY_C_LC.KY_C_UC/*.KY_L_LC.KY_L_UC*/; 
$front = 'өүӨҮ'/*.'öüÖÜ'*/; 
$excludes = 'еэгғкқЕЭГҒКҚ'/*.'eégğkqEÉGĞKQ'*/; 
// split text to words 
$matches = preg_split( '/[\b\s\-\.:]+/', $text, -1, PREG_SPLIT_OFFSET_CAPTURE); 
$mstart = 0; 
$ret = ''; 
foreach( $matches as $m ) { 
$ret .= substr( $text, $mstart, $m[1] - $mstart ); 
// is matched the word to front vowels? 
// exclude a words matched to е, э, г, к, к, қ, 
// them should be without hamza 
//if ( preg_match('/['.$front.']/u', $m[0]) && !preg_match('/['.$excludes.']/u', $m[0]) ) { 
//$ret .= preg_replace('/['.$letters.']+/u', H_HAMZA.'$0', $m[0]); 
//} else { 
$ret .= $m[0]; 
//} 
$mstart = $m[1] + strlen($m[0]); 
} 
$text =& $ret; 
foreach( $this->mCyLa2Arab as $pat => $rep ) { 
$text = preg_replace( $pat, $rep, $text ); 
} 
return $text; 
break; 
case 'ky-latn': 
case 'ky-tr': 
foreach( $this->mCyrl2Latn as $pat => $rep ) { 
$text = preg_replace( $pat, $rep, $text ); 
} 
return $text; 
break; 
case 'ky-cyrl': 
case 'ky-kgz': 
foreach( $this->mLatn2Cyrl as $pat => $rep ) { 
$text = preg_replace( $pat, $rep, $text ); 
} 
return $text; 
break; 
default: 
return $text; 
} 
} 

/* 
* We want our external link captions to be converted in variants, 
* so we return the original text instead -{$text}-, except for URLs 
*/ 
function markNoConversion( $text, $noParse=false ) { 
if( $noParse || preg_match( "/^https?:\\/\\/|ftp:\\/\\/|irc:\\/\\//", $text ) ) 
return parent::markNoConversion( $text ); 
return $text; 
} 

function convertCategoryKey( $key ) { 
return $this->autoConvert( $key, 'ky' ); 
} 

} 

/** 
* class that handles Cyrillic, Latin and Arabic scripts for Kyrgyz 
* right now it only distinguish ky_cyrl, ky_latn, ky_arab and ky_kgz, ky_tr, ky_cn. 
* 
* @ingroup Language 
*/ 
class LanguageKy extends LanguageKk_cyrl { 

function __construct() { 
global $wgHooks; 
parent::__construct(); 

$variants = array( 'ky', 'ky-cyrl', 'ky-latn', 'ky-arab', 'ky-kgz', 'ky-tr', 'ky-cn' ); 
$variantfallbacks = array( 
'ky' => 'ky-cyrl', 
'ky-cyrl' => 'ky', 
'ky-latn' => 'ky', 
'ky-arab' => 'ky', 
'ky-kgz' => 'ky-cyrl', 
'ky-tr' => 'ky-latn', 
'ky-cn' => 'ky-arab' 
); 

$this->mConverter = new KkConverter( $this, 'ky', $variants, $variantfallbacks ); 

$wgHooks['ArticleSaveComplete'][] = $this->mConverter; 
} 

/** 
* Work around for right-to-left direction support in ky-arab and ky-cn 
* 
* @return bool 
*/ 
function isRTL() { 
$variant = $this->getPreferredVariant(); 
if ( $variant == 'ky-arab' || $variant == 'ky-cn' ) { 
return true; 
} else { 
return parent::isRTL(); 
} 
} 

/* 
* It fixes issue with ucfirst for transforming 'i' to 'İ' 
* 
*/ 
function ucfirst ( $string ) { 
$variant = $this->getPreferredVariant(); 
if ( ($variant == 'ky-latn' || $variant == 'ky-tr') && $string[0] == 'i' ) { 
$string = 'İ' . substr( $string, 1 ); 
} else { 
$string = parent::ucfirst( $string ); 
} 
return $string; 
} 

/* 
* It fixes issue with lcfirst for transforming 'I' to 'ı' 
* 
*/ 
function lcfirst ( $string ) { 
$variant = $this->getPreferredVariant(); 
if ( ($variant == 'ky-latn' || $variant == 'ky-tr') && $string[0] == 'I' ) { 
$string = 'ı' . substr( $string, 1 ); 
} else { 
$string = parent::lcfirst( $string ); 
} 
return $string; 
} 

function convertGrammar( $word, $case ) { 
wfProfileIn( __METHOD__ ); 

$variant = $this->getPreferredVariant(); 
switch ( $variant ) { 
case 'ky-arab': 
case 'ky-cn': 
$word = parent::convertGrammarKk_arab( $word, $case ); 
break; 
case 'ky-latn': 
case 'ky-tr': 
$word = parent::convertGrammarKk_latn( $word, $case ); 
break; 
case 'ky-cyrl': 
case 'ky-kgz': 
case 'ky': 
default: 
$word = parent::convertGrammarKk_cyrl( $word, $case ); 
} 

wfProfileOut( __METHOD__ ); 
return $word; 
} 
} 
?>
